# *************************************************************************
#  SCRIPT FILE FOR: R EXAMPLES FOR ACTUARIES V0.1
#  Nigel De Silva (nigel.desilva@thomasmiller.com)
#  28 January 2006
# *************************************************************************

# *************************************************************************
#  NOTE: Scripts assume that data files are stored in C:\R\
#        Please change scripts accordingly of you have saved your files
#        elsewhere.
# *************************************************************************

clms <- read.csv("C:\\R\\sanders.csv", header=F)

plot(weld$x, weld$y)

library(MASS)


# ******************************************
# 3.1  R as a Calculator
# ******************************************

x <- pi*seq(0,4, by=0.1)
sin(x)

plot(sin, xlim=c(0,4*pi), main="The Sine Function")
points(x, sin(x), col="red")

uniroot(sin, c(3,5))


# ******************************************
# 3.2  R as a Calculator
# ******************************************

mammals <- read.csv("C:\\R\\mammals.csv")

dim(mammals)	# Find size of data frame
mammals[1:5, ]	# View the first five rows
summary(mammals)	# Get simple summary

mammals[is.na(mammals$brain), ]			# Look at missing data
mammals <- mammals[!is.na(mammals$brain), ]	# Remove missing data
dim(mammals)						# Check size of resulting data frame

attach(mammals)
plot(body, brain, main="Body Weight vs Brain Weight", 
  xlab="Body Weight (kg)", ylab="Brain Weight (g)")
identify(body, brain, rownames(mammals))	# Left click on data points to identify, right click to stop

mammals[c(1,5,25), ]

mammals <- mammals[-25, ]			# Negative subscript means "except" row 
mammals$lbody <- log(mammals$body)		# Create new vectors of logarithms 
mammals$lbrain <- log(mammals$brain)
attach(mammals)					# Reattach data frame as now altered

plot(lbody, lbrain, main="Body Weight vs Brain Weight", 
  xlab="log(Body Weight)", ylab="log(Brain Weight)")
identify(lbody, lbrain, rownames(mammals))	

mammals.lm <- lm(lbrain ~ lbody)
abline(mammals.lm, col=2)



# ******************************************
# 3.3  Exploring Distributions
# ******************************************

n <- 100000
sample <- rgamma(n, rate=0.5, shape=5)

library(MASS)
truehist(sample, main="Histogram of sample")

truehist(log(sample), main="Histogram of log(sample)")

boxcox(sample ~ 1)

fitdistr(sample, "Weibull")

# Fit distributions
fit.w <- fitdistr(sample, "Weibull")
fit.g <- fitdistr(sample, "gamma")

# Caclulate points for density curves
x <- seq(from=0, to=40, by=1)
y.w <- dweibull(x, shape=fit.w$estimate[1], scale=fit.w$estimate[2])
y.g <- dgamma(x, shape=fit.g$estimate[1], rate=fit.g$estimate[2])

# Draw histogram and density curves
truehist(sample, main="Histogram of sample")
lines(x, y.w, col=2)
lines(x, y.g, col=3) 
legend(30, 0.1, legend=c("Weibull distn", "Gamma distn"), lwd=1,
  col=c(2, 3))

# Draw cumulative density functions
plot(ecdf(sample), cex=0)
curve(pweibull(x, shape=fit.w$estimate[1], scale=fit.w$estimate[2]),
  add=T, col=2)
curve(pgamma(x, shape=fit.g$estimate[1], rate=fit.g$estimate[2]),
  add=T, col=3)
legend(30, 0.6, legend=c("Empirical CDF", "Weibull CDF", "Gamma CDF"), lwd=1,
  col=1:3)

#QQ-Plot
library(car)
par(mfrow = c(1, 2))	# Plot two graphs side by side

# First graph
qq.plot(sample, distribution="weibull", 
  shape=fit.w$estimate[1], scale=fit.w$estimate[2])
title(main="QQ plot of sample against \n Weibull quantiles", cex.main=0.9)

# Second graph
qq.plot(sample, distribution="gamma", 
  shape=fit.g$estimate[1], rate=fit.g$estimate[2])
title(main="QQ plot of sample against \n Gamma quantiles", cex.main=0.9)


# ******************************************
# 3.4  Fitting an extreme value distribution
# ******************************************

# Load up libraries to be used in analysis
library(evir)
library(evd)
library(MASS)

# Load up data
clms <- read.csv("C:\\R\\sanders.csv")
clms <- clms[,1]		# Want data in vector form

# Plot histogram
truehist(clms, nbins=20, main="Histogram of Large Claims Data")
meplot(clms)

# Mean Excess Plot 
meplot(clms)	# From evir package
mrlplot(clms)	# From evd package

# Number of points exceeding threshold
length(clms[clms>1000000])
length(clms[clms>400000])

# Fit linear regression line (fitted to points < 1m)
# to mean excess plot
me <- u <- sort(clms)[-length(clms)]
for (i in 1:(length(clms)-1)) {
  data <- clms[clms > u[i]]
  me[i] <- mean(data - u[i])
}
fit <- lm(me ~ u, subset=u<1000000)
mrlplot(clms)
abline(fit, col=2)

# Fit GPD
gpd.model <- gpd(clms, 406000)	# From evir package
gpd.model$par.ests

# QQ plot
qplot(clms, xi=gpd.model$par.ests[1])
library(car)				# Alternative with envelope
qqp(clms, distribution="gpd")		# Line is fitted, not theoretical
						# Used to test distribution, rather than fit

# CDF Plot
plot(ecdf(clms), cex=0, main="CDF Plot")
lines(sort(clms), pgpd(sort(clms), xi=gpd.model$par.ests[1], 
  beta=gpd.model$par.ests[2], mu=406000), col=2)

# 1 in 200 worst case
gpd.q(tailplot(gpd.model), 0.995)



# ******************************************
# 3.4  GLM:  Stochastic Reserving
# ******************************************

# Load up data
data <- read.csv("Mack.csv", header=F)
data <- as.matrix(data)

# Prepare data in correct form	
claims <- as.vector(data)
n.origin <- nrow(data)
n.dev <- ncol(data)
origin <- factor(row <- rep(1:n.origin, n.dev))
dev <- factor(col <- rep(1:n.dev, each=n.origin))

# Put into a data frame (no need, but easier to visualise)
mack <- data.frame(claims=claims, origin=origin, dev=dev)
mack[1:5, ]

# New quasi-poisson family 
quasipoisson <- function (link = "log")
## Amended by David Firth, 2003.01.16, at points labelled ###
## to cope with negative y values
##
## Computes Pearson X^2 rather than Poisson deviance
##
## Starting values are all equal to the global mean
{
     linktemp <- substitute(link)
     if (!is.character(linktemp)) {
         linktemp <- deparse(linktemp)
         if (linktemp == "link")
             linktemp <- eval(link)
     }
     if (any(linktemp == c("log", "identity", "sqrt")))
         stats <- make.link(linktemp)
     else stop(paste(linktemp, "link not available for poisson",
         "family; available links are", "\"identity\", \"log\" and \"sqrt\""))
     variance <- function(mu) mu
     validmu <- function(mu) all(mu > 0)
     dev.resids <- function(y, mu, wt) wt*(y-mu)^2/mu   ###
     aic <- function(y, n, mu, wt, dev) NA
     initialize <- expression({
         n <- rep(1, nobs)
         mustart <- rep(mean(y), length(y))             ###
     })
     structure(list(family = "quasipoisson", link = linktemp,
         linkfun = stats$linkfun, linkinv = stats$linkinv, variance = variance,
         dev.resids = dev.resids, aic = aic, mu.eta = stats$mu.eta,
         initialize = initialize, validmu = validmu, valideta = stats$valideta),
         class = "family")
}


# Fit model
model <- glm(claims ~ origin + dev, family = quasipoisson(), 
  subset=!is.na(claims))

# Extract useful info from the model
coef <- model$coefficients				# Get coefficients
disp <- summary(model)$dispersion			# Get dispersion parameter
cov.param <- disp * summary(model)$cov.unscaled	# Get covariance matrix of parameters

# To determine future uncertainty, need to create a 
# design matrix for future payments. Build up in stages.  
# Assume start from bottom left of future triangle.
n.fut.points <- length(claims[is.na(claims)])
fut.design <- matrix(0, nrow = n.fut.points, ncol=length(coef)) 	
fut.points <- claims
fut.points[!is.na(claims)] <- 0
fut.points[is.na(claims)] <- 1:n.fut.points
for(p in 1:n.fut.points){
  # All points and a constant in the predictor
  fut.design[p, 1] <- 1 				
  # Row factor
  fut.design[p, 1 + 
    as.numeric(origin[match(p, fut.points)]) - 1] <- 1
  # Col factor
  fut.design[p, 1 + (n.origin-1) + 
    as.numeric(dev[match(p, fut.points)]) - 1] <- 1
} 	
	

# Determine fitted future values (as a diagonal matrix)
fitted.values <- diag(as.vector(exp(fut.design %*% coef)))
total.reserve <- sum(fitted.values)
	
# Determine covariance matrix of linear predictors
cov.pred <- fut.design %*% cov.param %*% t(fut.design)	
	
# Determine covariance matrix of fitted values
cov.fitted <- fitted.values %*% cov.pred %*% fitted.values				

# Determine uncertainty statistics
total.rmse <- sqrt(disp*total.reserve+sum(cov.fitted))
total.predictionerror <- round(100*total.rmse/total.reserve)

total.rmse
total.predictionerror

